/* Multitaper header files ACO 4/00 */

#ifndef _MT_H
#define _MT_H

#ifndef __CINT__
/*  We Require that all code be Posix-compliant */
#ifndef _POSIX_SOURCE /* Avoid redefinition */
#define _POSIX_SOURCE
#endif
#endif

#include <iostream>

#ifdef __cplusplus /* Explicit C linkage is required in C++ code */
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <limits.h>
#include <time.h>
  // #include <stdarg.h>
    
#ifndef M_PI
#define M_PI            3.14159265358979323846
#endif
    
#ifndef M_PI_2
#define M_PI_2          1.57079632679489661923
#endif


// Descriptor class used in multitaper for removing spectral lines 
class RemovedLines {
 public:
  RemovedLines() : index(0) {}
  ~RemovedLines () {}
  int index;      // bin of spectral line removed 
  float fvalue;   // value of F-test for corresponding line 
  float re;       // real part of Fourier coefficient C removed 
  float im;       // imaginary part of Fourier coefficient C removed 
  float dcdbr;	   // real part of dC/dbin 
  float dcdbi;	   // imag part of dC/dbin 
  float d2cdb2r;  // real part of d^2C/dbin^2 
  float d2cdb2i;  // imag part of d^2C/dbin^2 
};

double fcrit(int npoints,int nwin);
int is_power_of_two(int test);
int index_cmp(const void *l1, const void *l2);
int fvalue_cmp(const void *l1, const void *l2);
int empty_window(float *workspace,int nbins,int index,int wwidth);
void sort_lines(RemovedLines *line_list,int *num_removed);
void remove_spectral_lines(float *data,int npoints,int padded_length,
   float nwdt,int nwin, int max_lines, int maxpass, int *num_removed, 
   RemovedLines *line_list,float *mtap_spec_init,float *mtap_spec_final,int dospec,int fimin, int fimax);
void mt_get_spec(float *series, int inum, int klength, float *amp);
void multitaper_spectrum(float *data, int npoints, int kind,
	int nwin, float nwdt, int inorm, float dt,
	float *ospec, float *dof, float *Fvalues, int klen,float *cest,int dospec);
void multitaper_cross_spectrum(float *data1,float *data2, int npoints, 
 			       int npadded, float dt, int nwin, float nwdt,
 			       double *spec12);
void slepian_tapers(int n, int nwin, double *el, float nwdt, double *tapers, double *tapsum);
int adwait(double *sqr_spec, double *dcf,
	double *el, int nwin, int num_freq,
	double *ares, double *degf, double avar);
void get_F_values2(double *sr, double *si, int nf, int nwin, float *Fvalue, double *b, float *cest);
void get_F_values(double *sri, int nf,
	     int nwin, float *Fvalue, double *b,float *cest,int imin,int imax);
int hires(double *sqr_spec, double *el, int nwin, int num_freq, double *ares);
double dmin(double a,double b);
double dmax(double a,double b);
int jtinvit(int *nm, int *n, double *d, double *e, double *e2,
	int *m, double *w, int *ind, double *z, int *ierr,
	double *rv1, double *rv2,
	double *rv3, double *rv4, double *rv6);
int jtridib(int *n, double *eps1, double *d, double *e, double *e2,
	double *lb, double *ub, int *m11, int *m,
	double *w, int *ind, int *ierr,
	double *rv4, double *rv5);
int get_pow_2(int inum);
double remove_mean(float*, int);
void zero_pad(float*,int,int);
void realft(float*,unsigned long,int);
void clear(float*,int,int);
void four1(float data[], unsigned long nn, int isign);
  void medfit(float x[], float y[], int ndata, float *a, float *b, float *abdev);
float rofunc(float b);
float NRselect(unsigned long k, unsigned long n, float arr[]);
int  mod_n(int x,int n);
float fmod_2pi(float x);

#ifdef __cplusplus
}
#endif   

#endif  // _MT_H
