//
//    File: MTLineMon.hh
//
#ifndef MTLINEMON_HH
#define MTLINEMON_HH


// const float fpi=3.14159265;


/*  We Require that all code be Posix-compliant */
#ifndef _POSIX_SOURCE /* Avoid redefinition */
#define _POSIX_SOURCE
#endif
class MTLineTable;

// DMT includes
#include "DatEnv.hh"
#include "MonServer.hh"
#include "Trend.hh" 
#include "OperStateCondList.hh"
#include "TrigPar.hh"
   
#include "DVector.hh"
#include "FSeries.hh"
#include "TSeries.hh"
#include "FSpectrum.hh"
#include "Complex.hh"

// ROOT includes
#include "TROOT.h"
#include "TApplication.h"
#include "TLine.h"
#include "TPaveText.h"
#include "TPaveLabel.h"
#include "TLatex.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TH1.h"
#include "TError.h"

namespace html {
  class document;
  class text;
}

#include "html/color.hh"
#include "html/document.hh"
#include "html/table.hh"
#include "html/size.hh"
#include "html/text.hh"
#include "html/writer.hh"

#include "MTUtility.hh"

//--------------------------------------  Channel descriptor class
class Chan {
public:
  Chan() {}
  ~Chan() {}
  TSeries* mTS;
  FSeries* mFS;
  char mName[32];
  double mRate;
  int mPoint;
};


// Descriptor class used in multitaper for removing spectral lines 
class RemovedLines {
 public:
  RemovedLines() : index(0) {}
  ~RemovedLines () {}
  int index;      // bin of spectral line removed 
  float fvalue;   // value of F-test for corresponding line 
  float dof;      // degrees of freedom for adaptive spectrum
  fComplex FC;    // the Fourier coefficient C removed  
  fComplex dFC;       // dC/dbin 
  fComplex d2FC;      // d^2C/dbin^2   
};

class compareIndex { // place in increasing order in frequency 
 public:
  int operator()(const RemovedLines& l1,const RemovedLines& l2) const 
    {return l1.index < l2.index; } 
};

class compareFValue { // place in decreasing order of significance
 public:
  int operator()(const RemovedLines& l1,const RemovedLines& l2) const 
    {return l1.fvalue > l2.fvalue; } 
};

class MTLineMon : public DatEnv, MonServer {
  friend class RemovedLines;
  public:
  MTLineMon(int argc, const char* argv[]);
  ~MTLineMon();
  void ProcessData(void);
  void Attention (void);
  //  inline int mod_n(const int x,const int n) { return  ( x >= 0 ? (x%n) : (n-1+((x+1)%n)) ); }

  private:
  // string serverName;
  std::string htmlDir;
  std::string trendDir;
  std::string outputDir;
  std::string trendFile;
  std::string indexFile;
  std::string filePath;
  std::string htmlFile;
  char lockStatus[256];

  std::string gpsTime;
  double currentTime;    // current GPS time stamp
  double startTime;      // start GPS time stamp
  double liveTime;       // live time
  double strideTime;     // stride




  void readConfigFile(const char* config);
  void usage();
  double fCrit();
  bool isPowerOfTwo(const int test);
  int getPowerOfTwo(const int test);
  bool emptyWindow(fComplex *workSpace,int index,int wwidth);
  void sortLines();
  void removeSpectralLines(const int fimin, const int fimax);
  //  void mt_get_spec(float *series, int inum, int klength, float *amp);
  void MTSpectrum(int inorm,
            float dt, FSpectrum *ospec, float *dof, float *Fvalues, fComplex *cEst);
  void slepianTapers(double *el, double *tapers, double *tapsum);
  int adaptive(FSpectrum **spec, double *dcf, double *el,
	     double *ares, double *degf, double avar);
  void getFValues(FSeries *f, float *Fvalue, double *tapsum, fComplex *cEst,int imin,int imax);
  void hiRes(FSpectrum **spec, double *lambda, FSpectrum *oSpec);

  void DumpHTML(const char* channel, const char* ofile); 
  
  inline int getDebug(void) const { return mDebug; }                    
  inline void setDebug(const int level) { mDebug = level; }            

  Chan chanv;

  int maxLines;  // maximum number of lines to remove
  int maxPasses;  // maximum number of passes through line removal loop

  enum {HIRES=1, ADAPTIVE}; // different types of multitaper spectrum
  int mSpectralType; // HIRES or ADAPTIVE

  enum {NO_CONFIG_FILE=1,INVALID_ARG,NO_LOCKLOSS,BAD_DATA}; // error codes

  int nWin;      // number of taper windows to use
  float nwdt;   // time-freq bandwidth 
  int paddingFactor;
  int paddedLength;
  float lowFreq,highFreq;  // frequency range to remove lines in 
  int mDump,nDump,maxDump;
  int mSegment; //which data seqment 
  int mMaxSegment; // maximum number of segments to process
  int nBuffer; // dmtviewer buffer size
  int decimationFactor;
  int lowBin,highBin;      // bin range to remove lines in 
  int nearNyquist;
  int wwidth;
  int nFreq;
  int nRemoved;
  std::vector<unsigned long> lineCount;

  float fCritical;
  FSpectrum *MTSpecInit;
  FSpectrum *MTSpecFinal;
  FSpectrum *SpecInit;
  FSpectrum *SpecFinal;
  TSeries *initialTS;
  TSeries *finalTS;
  double sampleTime;  // sample time in seconds
  double norm;
  std::vector<RemovedLines> lineList;
  bool mGetFinalSpec;
  bool mWriteLogFiles;
  bool mProduceTrends;
  bool mSnapshot; // look at just a single section
  bool first;
  int mDebug;


  Trend mTrend;
  OperStateCondList mOSC;
  TrigPar  mDict;
  int appendMode;

  MTLineTable* lmonTable;
};



#endif    // MTLINEMON_HH

