#include <TidEvent.h>
#include <stdarg.h>
#include <string.h>
#include <list>
#include <iostream>

using namespace std;

TidEvent::TidEvent(double alpha_, unsigned int argc, vector<string>& argv) {
  // np is number of parameters, not size of argv
  // argv should be list of variables/constants followed by list
  // of range (low bnd, up bnd, # points) for all the variables used
  unsigned int arcnt = 0;

  list<char> var; // variables

  alpha = alpha_;


  for(unsigned int i=0; i<argc; i++) {

    char *buf = (char *)(argv[arcnt].c_str());
    arcnt++;

    if(strspn(buf,"0123456789.-eE") == strlen(buf)) { // check if only numerics
      constants.push_back(atof(buf));
      params.push_back(constants.size()-1);

      addFactor.push_back(0.0);
      mulFactor.push_back(1.0);
    }
    else {

      int StrLen = (int)strlen(buf);

      if(strlen(buf)==1) // simple, single variable case
	if((*buf >= 97 && *buf <=122) ||
	   (*buf >= 65 && *buf <=90)) {
	  list<char>::iterator pc;
	  unsigned int j;
	  for(pc=var.begin(), j=0; pc!=var.end(); pc++, j++) 
	    if(*pc == *buf) break;
	  if(pc!=var.end()) {
	    params.push_back(j + 1000);
	    addFactor.push_back(0.0);
	    mulFactor.push_back(1.0);
	  }
	  else {
	    var.push_back(*buf);

	    varname.push_back(*buf);
	    varlb.push_back(0);
	    varub.push_back(0);
	    dvar.push_back(0);
      
	    params.push_back(dvar.size()+999);
	    addFactor.push_back(0.0);
	    mulFactor.push_back(1.0);
	  }
	}
	else {
	  cerr<<"Error, illegal variable name in TidEvent::TidEvent\n";
	  exit(-1);
	}
      else { // more complex case with multiplier and offset
	char *now = strchr(buf,'*');
	if(!now) {
	  now = buf;
	  mulFactor.push_back(1.0);
	}
	else {
	  *now = 0;
	  mulFactor.push_back(atof(buf));
	  now++;
	}

	if((*now < 97 || *now > 122) &&
	   (*now < 65 || *now > 90)) {
	  cerr<<"Error, illegal variable name in TidEvent::TidEvent\n";
	  exit(-1);
	}

	unsigned int j;
	list<char>::iterator pc;
	for(pc=var.begin(), j=0; pc!=var.end(); pc++, j++) 
	  if(*pc == *now) break;
	if(pc!=var.end()) params.push_back(j+1000);
	else {
	  var.push_back(*now);

	  varname.push_back(*now);
	  varlb.push_back(0);
	  varub.push_back(0);
	  dvar.push_back(0);
      
	  params.push_back(dvar.size()+999);
	}

	now++;
	if(now-buf <= StrLen) {
	  if(*now=='-') {
	    addFactor.push_back(atof(now));
	  }
	  else {
	    if(*now=='+') {
	      now++;
	      addFactor.push_back(atof(now));
	    }
	    else {
	      cerr<<"Error: syntax error in TidEvent::TidEvent\n";
	      exit(-1);
	    }
	  }
	}
	else addFactor.push_back(0);
      }
    }
  }

  
  list<char>::iterator p;
  unsigned int k, l;

  for(p=var.begin(), l=0; p!=var.end(); p++, l++) {
    for(k=argc; k<argv.size(); k+=4)
      if(*((char*)argv[k].c_str()) == *p) break;
    if(k==argv.size()) {
      cerr<<"Error: variable "<<*p<<" has no range information.\n";
      exit(-1);
    }
    else {
      varlb[l] = atof(argv[k+1].c_str());
      varub[l] = atof(argv[k+2].c_str());
      dvar[l] = (varub[l] - varlb[l]) / (atof(argv[k+3].c_str())-1.0);
    }
  }

}


/*
double TidEvent::Cost(ArrayBool *) {

  bool DoIt;
  double min, mim;

  vector<double> par;

  // initialize
  for(unsigned int i=0; i<params.size(); i++) {
    if(params[i] >= 1000) par[i]=addFactor[params[i]-1000] + mulFactor[params[i]-1000]*varlb[params[i]-1000];
    else par[i] = constants[params[i]];
  }
  min = CostFunction(data, &par);

  DoIt=1;
  while(DoIt) {

    for(int i=par.size()-1; i>=0; i--) 
      if(params[i] >= 1000) {
	if(par[i] >= addFactor[params[i]-1000] + mulFactor[params[i]-1000]*varub[params[i]-1000]) {
	  if(i==0) {DoIt=0; break;}
	  else
	    par[i]=addFactor[params[i]-1000] + mulFactor[params[i]-1000]*varlb[params[i]-1000];
	}
	else par[i] += mulFactor[params[i]-1000]*dvar[params[i]-1000];
      }

    mim = CostFunction(data, &par);

    if(mim < min) min = mim;
  }
			   
  return min;
}
*/


double TidEvent::GetParamMin (unsigned int i) {

  if(params[i]<1000) return constants[params[i]];
  
  else return addFactor[i] + mulFactor[i] * varlb[params[i]-1000];

}


double TidEvent::GetParamMax (unsigned int i) {

  if(params[i]<1000) return constants[params[i]];
  
  else return addFactor[i] + mulFactor[i] * varub[params[i]-1000];

}
