/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "FrStatDataRef.hh"
#include "framecpp/FrStatData.hh"
#include "FrVectRef.hh"
#include "Time.hh"

using namespace std;

//=====================================  Default constructor
FrStatDataRef::FrStatDataRef(void)
  : mOwner(false), mData(0)
{}

//=====================================  Default constructor
FrStatDataRef::FrStatDataRef(const stat_type* x)
  : mOwner(false), mData(x)
{}

//=====================================  Default constructor
FrStatDataRef::FrStatDataRef(const string& name, const string& comment, 
		       const string& repr, long vsn, const Time& tStart, 
		       const Time& tEnd, void* det, FrVectRef& v) 
  : mOwner(false), mData(0)
{
    setStat(name, comment, repr, vsn, tStart, tEnd, det, v);
}

//=====================================  Destructor
FrStatDataRef::~FrStatDataRef(void) {
    setStat(0,false);
}

//=====================================  Get reference to the data vector
FrVectRef
FrStatDataRef::getFrVect(void) const {
    return FrVectRef(mData->RefData()[0]);
}

//=====================================  Get the end time
Time
FrStatDataRef::getEndTime(void) const {
    return Time(mData->GetTimeEnd());
}

//=====================================  get the name
const char*
FrStatDataRef::getName(void) const {
    return mData->GetName().c_str();
}

//=====================================  get the name
const char*
FrStatDataRef::getRepresentation(void) const {
    return mData->GetRepresentation().c_str();
}

//=====================================  get the start time
Time
FrStatDataRef::getStartTime(void) const {
    return Time(mData->GetTimeStart());
}

//=====================================  Get the version number
long
FrStatDataRef::getVersion(void) const {
    return mData->GetVersion();

}

//=====================================  Default constructor
FrStatDataRef::stat_type*
FrStatDataRef::replicate(void) const {
    return new stat_type(*mData);
}

//=====================================  Default constructor
void
FrStatDataRef::setStat(const string& name, const string& comment, 
		       const string& repr, long vsn, const Time& tStart, 
		       const Time& tEnd, void* det, FrVectRef& v) {

    stat_type* s = new stat_type(name, comment, repr, tStart.getS(),
				 tEnd.getS(), vsn);

    s->RefData().append(v.release(), false, true);
    s->SetDetector( reinterpret_cast<FrameCPP::FrDetector*>(det) );
    setStat(s, true);
}

//=====================================  Default constructor
void
FrStatDataRef::setStat(stat_type* v, bool owner) {
    if (mOwner) delete mData;
    mData  = v;
    mOwner = owner;
}
